@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinMenu.c,v 1.59.2.1 2009/02/23 10:37:38 patthoyts Exp $
+ * RCS: @(#) $Id: tkWinMenu.c,v 1.59.2.3 2009/09/14 23:40:42 hobbs Exp $
  */
 
 #define OEMRESOURCE
@@ -50,6 +50,10 @@
 #define MENU_SYSTEM_MENU		MENU_PLATFORM_FLAG1
 #define MENU_RECONFIGURE_PENDING	MENU_PLATFORM_FLAG2
 
+#ifndef WM_UNINITMENUPOPUP
+#define WM_UNINITMENUPOPUP              0x0125
+#endif
+
 static int indicatorDimensions[2];
 				/* The dimensions of the indicator space in a
 				 * menu entry. Calculated at init time to save
@@ -992,6 +996,19 @@ TkWinHandleMenuEvent(
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     switch (*pMessage) {
+    case WM_UNINITMENUPOPUP:
+	hashEntryPtr = Tcl_FindHashEntry(&tsdPtr->winMenuTable,
+		(char *) *pwParam);
+	if (hashEntryPtr != NULL) {
+	    menuPtr = (TkMenu *) Tcl_GetHashValue(hashEntryPtr);
+	    if ((menuPtr->menuRefPtr != NULL)
+		    && (menuPtr->menuRefPtr->parentEntryPtr != NULL)) {
+		TkPostSubmenu(menuPtr->interp,
+			menuPtr->menuRefPtr->parentEntryPtr->menuPtr, NULL);
+	    }
+	}
+	break;
+
     case WM_INITMENU:
 	TkMenuInit();
 	hashEntryPtr = Tcl_FindHashEntry(&tsdPtr->winMenuTable,
@@ -1775,8 +1792,7 @@ DrawMenuEntryArrow(
     COLORREF oldBgColor;
     RECT rect;
 
-    if (!drawArrow || (mePtr->type != CASCADE_ENTRY)
-	    || (mePtr->state != ENTRY_DISABLED)) {
+    if (!drawArrow || (mePtr->type != CASCADE_ENTRY)) {
 	return;
     }
 
@@ -1795,7 +1811,8 @@ DrawMenuEntryArrow(
 	gc->background = activeBgColor->pixel;
     }
 
-    gc->foreground = GetSysColor(COLOR_GRAYTEXT);
+    gc->foreground = GetSysColor((mePtr->state == ENTRY_DISABLED) ?
+	    COLOR_GRAYTEXT : COLOR_MENUTEXT);
 
     rect.top = y + GetSystemMetrics(SM_CYBORDER);
     rect.bottom = y + height - GetSystemMetrics(SM_CYBORDER);
@@ -1981,14 +1998,15 @@ TkWinMenuKeyObjCmd(
 	    virtualKey = XKeysymToKeycode(winPtr->display, keySym);
 	    scanCode = MapVirtualKey(virtualKey, 0);
 	    if (0 != scanCode) {
+		XKeyEvent xkey = eventPtr->xkey;
 		CallWindowProc(DefWindowProc, Tk_GetHWND(Tk_WindowId(tkwin)),
 			WM_SYSKEYDOWN, virtualKey,
 			(int) ((scanCode << 16) | (1 << 29)));
-		if (eventPtr->xkey.nbytes > 0) {
-		    for (i = 0; i < eventPtr->xkey.nbytes; i++) {
+		if (xkey.nbytes > 0) {
+		    for (i = 0; i < xkey.nbytes; i++) {
 			CallWindowProc(DefWindowProc,
 				Tk_GetHWND(Tk_WindowId(tkwin)), WM_SYSCHAR,
-				eventPtr->xkey.trans_chars[i],
+				xkey.trans_chars[i],
 				(int) ((scanCode << 16) | (1 << 29)));
 		    }
 		}
